# Copyright 2018-2025 Marek Susčík <markuv.koutek@seznam.cz>
# This add-on is free software, licensed under the terms of the GNU General Public License (version 2). see <https://www.gnu.org/licenses/gpl-2.0.html>.

from .xml.etree import ElementTree as ET
import os.path
from logHandler import log
import wx
import gui
from gui import guiHelper
import locale
import json
import ui
from contextlib import closing
from urllib.request import urlopen
try:
  from . import vlc
except:
  vlcInstalled = False
else:
  vlcInstalled = True

locale.setlocale(locale.LC_ALL, "czech")
if vlcInstalled:
  instance = vlc.Instance('--input-repeat=-1', '--fullscreen')
  player = instance.media_player_new()

class RadioGroupsDialog(wx.Frame):

  allowEnter = False
  
  _instance = None
  def __new__(cls, *args, **kwargs):
    # Make this a singleton.
    if RadioGroupsDialog._instance is None:
      return super(RadioGroupsDialog, cls).__new__(cls, *args, **kwargs)
    return RadioGroupsDialog._instance

  def __init__(self, parent):
    if RadioGroupsDialog._instance is not None:
      return
    RadioGroupsDialog._instance = self

    # The title of a dialog
    super(RadioGroupsDialog, self).__init__(parent, title="Rádio - zvolte kategorii")

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)
    self.Bind(wx.EVT_CLOSE, self.onClose)

    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
    
    # menu items
    item = menu.Append(wx.ID_ANY, "Zobrazit &seznam stanic\tEnter")
    self.Bind(wx.EVT_MENU, self.onGroupsListChoice, item)
    item = menu.Append(wx.ID_ANY, "&Zapnout nebo vypnout přehrávání aktuální stanice\tSpace")
    self.Bind(wx.EVT_MENU, self.stationPlayStopItem, item)
    item = menu.Append(wx.ID_ANY, "Zesí&lit rádio\tCtrl++")
    self.Bind(wx.EVT_MENU, self.radioVolumeUpItem, item)
    item = menu.Append(wx.ID_ANY, "Zesl&abit rádio\tCtrl+-")
    self.Bind(wx.EVT_MENU, self.radioVolumeDownItem, item)
    item = menu.Append(wx.ID_ANY, "Nápo&věda")
    self.Bind(wx.EVT_MENU, self.helpItem, item)
    menu.AppendSeparator()
    item = menu.Append(wx.ID_EXIT, "U&končit")
    self.Bind(wx.EVT_MENU, self.onClose, item)
    
    # the menu title
    menuBar.Append(menu, "Rádio")
    self.SetMenuBar(menuBar)

    # get radio categories from XML
    xmlUrl = "https://suscik.cz/nvda/oko/radiooko.xml"
    offlinedb = False
    try:
      response = urlopen(xmlUrl).read()
    except:
      offlinedb = True
    if not offlinedb:
      tree = ET.fromstring(response)
    else:
      xmlFile = os.path.join(os.path.dirname(__file__), "files\\radio\\radio.xml")
      treexml = ET.parse(xmlFile)
      tree = treexml.getroot()
      self.SetTitle("Rádio - zvolte kategorii (místní databáze)")

    # listbox with station categories
    self.choices = []
    self.choices = [x.attrib["name"] for x in tree]
    self.choices.sort(key=locale.strxfrm)
    self.groupsList = wx.ListBox(self, choices=self.choices)
    self.groupsList.Selection = 0
    self.groupsList.Bind(wx.EVT_LISTBOX_DCLICK, self.onGroupsListChoice)
    self.groupsList.Bind(wx.EVT_KEY_UP, self.onGroupsListKeyDown)
    sHelper.addItem(self.groupsList, flag = wx.EXPAND)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    self.Sizer = mainSizer

    self.groupsList.SetFocus()
    self.CentreOnScreen()
    self.allowEnter = False

    # the timer which don't allow react to Enter key shortly after running this module
    self.timer = wx.Timer(self)
    self.timer.StartOnce(100)
    self.Bind(wx.EVT_TIMER, self.enoughRuntime)


  def enoughRuntime(self, evt):
    self.allowEnter = True

  def __del__(self):
    RadioGroupsDialog._instance = None

  def helpItem(self, evt):
    self.allowEnter = False
    from . import addonDoc
    os.startfile(addonDoc.getDocPath("radio.html"))

  def stationPlayStopItem(self, evt):
    if radio.isPlaying(self):
      radio.stop(self)
    else:
      radio.play(self)

  def radioVolumeUpItem(self, evt):
    Radio.changeVolume(self, 10)
  
  def radioVolumeDownItem(self, evt):
    Radio.changeVolume(self, -10)
  
  def onGroupsListChoice(self, evt):
    self.Hide()
    self.Disable()
    station.category = self.groupsList.StringSelection
    try:
      RadioStationsDialog(self).Show()
    except Exception as e:
      self.Enable()
      raise e

  def onGroupsListKeyDown(self, evt):
    key = evt.GetKeyCode()
    if key == key == wx.WXK_NUMPAD_ENTER:
      if not self.allowEnter:
        self.allowEnter = True
        return
      else:
        self.onGroupsListChoice(self)
    elif key == 388: # num +
      if evt.ControlDown():
        self.radioVolumeUpItem(None)
    elif key == 390: # num -
      if evt.ControlDown():
        self.radioVolumeDownItem(None)
    elif key == wx.WXK_ESCAPE:
      self.onClose(self)
    evt.Skip()

  def onClose(self, evt):
    global radioConfig
    if bool(radioConfig):
      radio.saveConfig(self)
    self.Destroy()
    RadioGroupsDialog._instance = None


class RadioStationsDialog(wx.Frame):

  stationsDict = dict()

  def __init__(self, parent):
    # The title of the stations dialog
    super(RadioStationsDialog, self).__init__(parent, size=(400, 800), title="Vyberte stanici")
    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
    
    # menu items
    item = menu.Append(wx.ID_ANY, "&Spustit přehrávání zvolené stanice\tEnter")
    self.Bind(wx.EVT_MENU, self.onStationsListChoice, item)
    item = menu.Append(wx.ID_ANY, "&Zapnout nebo vypnout přehrávání aktuální stanice\tSpace")
    self.Bind(wx.EVT_MENU, self.Parent.stationPlayStopItem, item)
    item = menu.Append(wx.ID_ANY, "Zesí&lit rádio\tCtrl++")
    self.Bind(wx.EVT_MENU, self.Parent.radioVolumeUpItem, item)
    item = menu.Append(wx.ID_ANY, "Zesl&abit rádio\tCtrl+-")
    self.Bind(wx.EVT_MENU, self.Parent.radioVolumeDownItem, item)
    item = menu.Append(wx.ID_ANY, "Zpět na &seznam kategorií\tBackspace")
    self.Bind(wx.EVT_MENU, self.backtoCategoriesItem, item)
    item = menu.Append(wx.ID_ANY, "Nápo&věda")
    self.Bind(wx.EVT_MENU, self.Parent.helpItem, item)
    menu.AppendSeparator()
    item = menu.Append(wx.ID_EXIT, "U&končit")
    self.Bind(wx.EVT_MENU, self.onClose, item)
    
    # the menu title
    menuBar.Append(menu, "Stanice")
    self.SetMenuBar(menuBar)

   # get stations list from xml
    xmlUrl = "https://suscik.cz/nvda/oko/radiooko.xml"
    offlinedb = False
    try:
      response = urlopen(xmlUrl).read()
    except:
      offlinedb = True
    if not offlinedb:
      tree = ET.fromstring(response)
    else:
      xmlFile = os.path.join(os.path.dirname(__file__), "files\\radio\\radio.xml")
      tree = ET.parse(xmlFile)

    self.stationsDict = [x.attrib for x in tree.find(f"Category[@name='{station.category}']")]

    # The stations list
    stationsChoices = [x["Description"] for x in self.stationsDict]
    stationsChoices.sort(key=locale.strxfrm)
    self.stationsList = wx.ListBox(self, choices=stationsChoices)
    self.stationsList.Selection = 0
    self.stationsList.Bind(wx.EVT_CHOICE, self.onStationsListChoice)
    self.stationsList.Bind(wx.EVT_LISTBOX_DCLICK, self.onStationsListChoice)
    self.stationsList.Bind(wx.EVT_KEY_UP, self.onStationsListKeyDown)
    sHelper.addItem(self.stationsList, flag = wx.EXPAND)

  def backtoCategoriesItem(self, evt):
    self.Parent.Enable()
    self.Parent.Show()
    self.Destroy()

  def onStationsListKeyDown(self, evt):
    key = evt.GetKeyCode()
    if key == key == wx.WXK_NUMPAD_ENTER:
      self.onStationsListChoice(self)
    elif key == 388: # num +
      if evt.ControlDown():
        self.Parent.radioVolumeUpItem(None)
    elif key == 390: # num -
      if evt.ControlDown():
        self.Parent.radioVolumeDownItem(None)
    elif key == wx.WXK_ESCAPE:
      self.onClose(self)
    evt.Skip()

  def onStationsListChoice(self, evt):
    station.name = self.stationsList.StringSelection
    for x in self.stationsDict:
      if x["Description"] == station.name:
        station.url = x["URL"]
        break
    radio.saveConfig(self)
    radio.play(self)    

  def onClose(self, evt):
    global radioConfig
    if bool(radioConfig):
      radio.saveConfig(self)
    self.Parent.Destroy()
    self.Destroy()
    RadioGroupsDialog._instance = None
 

class Radio:

  isAllowed = vlcInstalled
  volume = 100

  def play(self):
    url = station.url
    if url:
      ui.message(station.name)
      media=instance.media_new(url)
      player.set_media(media)
      player.stop()
      player.audio_set_volume(radio.volume)
      player.play()
    else:
      ui.message("Nebylo vybráno žádné rádio.")

  def stop(self):
    player.stop()

  def pause(self):
    player.pause()

  def isPlaying(self):
    return player.is_playing() 

  def changeVolume(self, volume):
    radio.volume = radio.volume + volume
    if radio.volume > 100:
      radio.volume = 100
    if radio.volume < 0:
      radio.volume = 0
    ui.message(str(radio.volume))
    player.audio_set_volume(radio.volume)

  def saveConfig(self):
    global radioConfig
    radioConfig['LastStationName'] = station.name 
    radioConfig['LastStationUrl'] = station.url
    radioConfig['LastStationCategory'] = station.category
    radioConfig['Volume'] = radio.volume
    SetFile = os.path.join(os.path.dirname(__file__), "files\\radio\\configradio.json")
    with open(SetFile, 'w') as f:
      json.dump(radioConfig, f)    


class Station:

  name = ""
  url = ""
  category = ""


radio = Radio
station = Station
radioConfig = dict()
station.url = ""
radio.volume = 100

#read settings
if os.path.isfile(os.path.join(os.path.dirname(__file__), "files\\radio\\configradio.json")):
  SetFile = os.path.join(os.path.dirname(__file__), "files\\radio\\configradio.json")
  with open(SetFile, 'r') as f:
    radioConfig = json.load(f)
  station.name = radioConfig['LastStationName']
  station.url = radioConfig['LastStationUrl']
  station.category = radioConfig['LastStationCategory']
  try:
    radio.volume = radioConfig['Volume']
  except:
    radio.volume = 100